local dewdrop = AceLibrary("Dewdrop-2.0")
local tablet = AceLibrary("Tablet-2.0")
local L = AceLibrary("AceLocale-2.0"):new("RegenFu")



RegenFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0")

local optionsTable = {
	type = 'group',
	args = {
		showHP = {
			order = 1,
			type = 'toggle',
			name = L["MENU_SHOW_HP"],
			desc = L["MENU_SHOW_HP"],
			set = "ToggleShowHP",
			get = "IsShowHP",
		},
		showMP = {
			order = 2,
			type = 'toggle',
			name = L["MENU_SHOW_MP"],
			desc = L["MENU_SHOW_MP"],
			set = "ToggleShowMP",
			get = "IsShowMP",
		},
		showPercent = {
			order = 3,
			type = 'toggle',
			name = L["MENU_SHOW_PERCENT"],
			desc = L["MENU_SHOW_PERCENT"],
			set = "ToggleShowPercent",
			get = "IsShowPercent",
		},
		showCurrent = {
			order = 4,
			type = 'toggle',
			name = L["MENU_SHOW_CURRENT"],
			desc = L["MENU_SHOW_CURRENT"],
			set = "ToggleShowCurrent",
			get = "IsShowCurrent",
		},
		hideLabel = {
			order = 5,
			type = 'toggle',
			name = L["MENU_HIDE_LABEL"],
			desc = L["MENU_HIDE_LABEL"],
			set = "ToggleHideLabel",
			get = "IsHideLabel",
		}
	}
}

RegenFu.OnMenuRequest = optionsTable
RegenFu:RegisterChatCommand( { "/regenfu" }, optionsTable )

RegenFu:RegisterDB("FuBar_RegenDB")
RegenFu:RegisterDefaults('profile', {
	showHP = true,
	showMP = true,
	showLabel = true,
})


	-- Methods
function RegenFu:IsShowHP()
	return self.db.profile.showHP;
end
	
function RegenFu:IsShowMP()
	return self.db.profile.showMP;
end
	
function RegenFu:IsShowCurrent()
	return self.db.profile.showCurrent;
end
	
function RegenFu:IsShowPercent()
	return self.db.profile.showPercent;
end
		
	
function RegenFu:IsHideLabel()
	return self.db.profile.hideLabel;
end
	
function RegenFu:ToggleShowHP()
	self.db.profile.showHP = not self.db.profile.showHP;
	self:UpdateSettings();
end
	
function RegenFu:ToggleShowMP()
	self.db.profile.showMP = not self.db.profile.showMP;
	self:UpdateSettings();
end

function RegenFu:ToggleShowCurrent()
	self.db.profile.showCurrent = not self.db.profile.showCurrent;
	self:UpdateSettings();
end
		
function RegenFu:ToggleShowPercent()
	self.db.profile.showPercent = not self.db.profile.showPercent;
	self:Update();
end
	
	
function RegenFu:ToggleHideLabel()
	self.db.profile.hideLabel = not self.db.profile.hideLabel;
	self:Update();
end
	
function RegenFu:OnInitialize()
	self:RegisterEvent("UNIT_HEALTH");
	self:RegisterEvent("UNIT_MANA");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("PLAYER_REGEN_DISABLED");
	self:RegisterEvent("PLAYER_REGEN_ENABLED");

	self.vars = {
		currHealth = 0,
		currMana = 0,
		regenHP = 0,
		regenMP = 0,
		checkedManaState = 0,
		maxHPRate = 0,
		minHPRate = 9999,
		maxMPRate = 0,
		minMPRate = 9999,
		regenCombatTrack = 0,
		regenMPDuringCombat = 0,
		regenHPDuringCombat = 0,
	}
	
end
	
		
function RegenFu:PLAYER_ENTERING_WORLD()
	if UnitManaMax("player") == 0 then
		self.db.profile.showMP = FALSE;
	end
end
	
function RegenFu:PLAYER_REGEN_DISABLED()
	self.vars.regenCombatTrack = 1;
	self.vars.regenMPDuringCombat = 0;
	self.vars.regenHPDuringCombat = 0;
end

function RegenFu:PLAYER_REGEN_ENABLED()
	self.vars.regenCombatTrack = 0;	
end

function RegenFu:UNIT_HEALTH()
	if self:IsShowHP() then
		local currHealth = UnitHealth("player");
		if ( currHealth > self.vars.currHealth and self.vars.currHealth  ~= 0 ) then
			self.vars.regenHP  = currHealth - self.vars.currHealth ;
			
			if ( self.vars.regenCombatTrack == 1) then
				self.vars.regenHPDuringCombat = self.vars.regenHPDuringCombat + self.vars.regenHP;
			end 
			
			if (self.vars.regenHP > self.vars.maxHPRate) then 
				self.vars.maxHPRate = self.vars.regenHP ;
			end
			if (self.vars.regenHP < self.vars.minHPRate or self.vars.minHPRate == 9999) then 
				self.vars.minHPRate = self.vars.regenHP;
			end				
		end
		self.vars.currHealth = currHealth;
		self:Update();
	end
end

function RegenFu:UNIT_MANA()
	if self:IsShowMP() then
		local currMana = UnitMana("player");
		if ( currMana  > self.vars.currMana and self.vars.currMana ~= 0 ) then
			self.vars.regenMP  = currMana - self.vars.currMana  ;

			if ( self.vars.regenCombatTrack == 1) then
				self.vars.regenMPDuringCombat = self.vars.regenMPDuringCombat + self.vars.regenMP;
			end 

			if (self.vars.regenMP > self.vars.maxMPRate) then 
				self.vars.maxMPRate = self.vars.regenMP;
			end
			if (self.vars.regenMP < self.vars.minMPRate or self.vars.minMPRate == 9999) then 
				self.vars.minMPRate = self.vars.regenMP;
			end								
		end
		self.vars.currMana = currMana;
		self:Update();
	end
end

function RegenFu:OnTextUpdate()
	local labelTextHP = "";
	local valueTextHP = "";
	local labelTextMP = "";
	local valueTextMP = "";
	
	if UnitHealth("player") == UnitHealthMax("player") then
		self.vars.regenHP = 0;
	end
	if UnitMana("player") == UnitManaMax("player") then
		self.vars.regenMP = 0;
	end	
			
	-- safety in case both are off, then cant ever turn em on
	if ( not self:IsShowHP() and not self:IsShowMP() ) then
		self:ToggleShowHP();
	end
	
	if ( self:IsShowHP() ) then
		if not self:IsHideLabel() then
			labelTextHP = L.HP_LABEL;
		end
		
		if self:IsShowPercent() then
			valueTextHP = string.format("%.2f%%", (self.vars.regenHP/UnitHealthMax("player"))*100);
		else
			valueTextHP = string.format("%d", self.vars.regenHP);	
		end	

		valueTextHP = string.format("|cff00ff00%s|r", valueTextHP);

		if self:IsShowCurrent() then
			valueTextHP = string.format("|cff00ff00%s|r/|cff00ff00%s|r(%s)", UnitHealth("player"), UnitHealthMax("player"), valueTextHP);
		end
		
	end
	
	if ( self:IsShowMP() ) then
		if not self:IsHideLabel() then
			labelTextMP = L.MP_LABEL;
		end
		
		if self:IsShowPercent() then
			valueTextMP = string.format("%.2f%%", (self.vars.regenMP/UnitManaMax("player"))*100);
		else
			valueTextMP = string.format("%d", self.vars.regenMP);	
		end
		
		valueTextMP = string.format("|cff3399ff%s|r", valueTextMP);

		if self:IsShowCurrent() then
			valueTextMP = string.format("|cff3399ff%s|r/|cff3399ff%s|r(%s)", UnitMana("player"), UnitManaMax("player"), valueTextMP)
		end

	end
	
	self:SetText(labelTextHP..valueTextHP.." "..labelTextMP..valueTextMP);
end
	
function RegenFu:OnTooltipUpdate()

	local minHP = self.vars.minHPRate;
	local minMP = self.vars.minMPRate;
	
	if minHP == 9999 then minHP = 0 end;
	if minMP == 9999 then minMP = 0 end;	

	local cat, regenPercent
	
	if self:IsShowHP() then
	
		regenPercent = (self.vars.regenHPDuringCombat/UnitHealthMax("player"))*100;
		
		cat = tablet:AddCategory(
			'columns', 2,
			'child_textR', 1,
			'child_textG', 1,
			'child_textB', 0,
			'child_text2R', 0,
			'child_text2G', 1,
			'child_text2B', 0
		);

		cat:AddLine(
			'text', L.TOOLTIP1_LEFT, 
			'text2', string.format("%s |cffffffff/|r %s |cffffffff(|r|cffff0000%s|cffffffff)|r", UnitHealth("player"), UnitHealthMax("player"), UnitHealthMax("player")-UnitHealth("player") )
		);
		cat:AddLine(
			'text', L.TOOLTIP3_LEFT, 
			'text2', self.vars.maxHPRate
		);
		cat:AddLine(
			'text', L.TOOLTIP4_LEFT, 
			'text2', minHP
		);
		cat:AddLine(
			'text', L.TOOLTIP8_LEFT, 
			'text2', string.format("%s (%.2f%%)", self.vars.regenHPDuringCombat, regenPercent )
		);			
	end
	
	if self:IsShowMP() then
	
		regenPercent = (self.vars.regenMPDuringCombat/UnitManaMax("player"))*100;
		
		cat = tablet:AddCategory(
			'columns', 2,
			'child_textR', 1,
			'child_textG', 1,
			'child_textB', 0,
			'child_text2R', 0.2,
			'child_text2G', 0.4,
			'child_text2B', 1
		);

		cat:AddLine(
			'text', L.TOOLTIP2_LEFT, 
			'text2', string.format("%s |cffffffff/|r %s |cffffffff(|r|cffff0000%s|cffffffff)|r", UnitMana("player"), UnitManaMax("player"), UnitManaMax("player")-UnitMana("player") )
		);
		cat:AddLine(
			'text', L.TOOLTIP5_LEFT, 
			'text2', self.vars.maxMPRate
		);
		cat:AddLine(
			'text', L.TOOLTIP6_LEFT, 
			'text2', minMP
		);
		cat:AddLine(
			'text', L.TOOLTIP7_LEFT, 
			'text2', string.format("%s (%.2f%%)", self.vars.regenMPDuringCombat, regenPercent )
		);
				
	end		
end
	
function RegenFu:UpdateSettings()
	-- safety in case both are off, then cant ever turn em on
	if ( not self:IsShowHP() and not self:IsShowMP() ) then
		self:ToggleShowHP();
	end
	self:Update();
end
