--------------------------------------------------------------------------------
-- GetLink Revision 3, bugfix/patch 10
--------------------------------------------------------------------------------
-- vars --
GLSavedVars = {
	max = 15; -- max to return for bot
	enabled = true; -- bot enabled
	ignored = {}; -- people to ignore on bot
	hiding = true; -- hide what you link to others automagically
	w = true; -- return links in whispers?
    trigger = "!gl";
}
GLWords = {
	qualities = {
		artifact = 6, legendary = 5, epic = 4, rare = 3,
		uncommon = 2, common = 1, poor = 0,
        orange = 5, purple = 4, blue = 3, green = 2,
        white = 1, grey = 0, gray = 0,
	},
};
SlashCmdList["GLBot"] = function(args)
    -- GetLink Bot slash command, options:
    --  on, off, ignore, max,
    --  hide, show, help,
    args = string.lower(args);
    if(string.sub(args, 1, 2) == "on") then
        GLSavedVars.enabled = true;
        GLPrint(GETLINK_CONFIG_ON);
    elseif(string.sub(args, 1, 3) == "off") then
        GLSavedVars.enabled = nil;
        GLPrint(GETLINK_CONFIG_OFF);
    elseif(string.sub(args, 1, 6) == "ignore") then
        local n = string.upper(string.sub(args, 8, 8)) .. string.sub(args, 9, -1);
        if(GLSavedVars.ignored[n]) then
            GLSavedVars.ignored[n] = nil;
            GLPrint(string.format(GETLINK_CONFIG_IGNORE_NO, n));
        else
            GLSavedVars.ignored[n] = true;
            GLPrint(string.format(GETLINK_CONFIG_IGNORE, n));
        end
    elseif(string.sub(args, 1, 3) == "max") then
        local n = tonumber(string.sub(args, 5, -1));
        if(type(n) ~= "number") then
            GLPrint(GETLINK_CONFIG_MAX_NUMBER);
        else
            GLSavedVars.max = n;
            GLPrint(string.format(GETLINK_CONFIG_MAX, n));
        end
    elseif(string.sub(args, 1, 4) == "hide") then
        GLSavedVars.hiding = true;
        GLPrint(GETLINK_CONFIG_HIDE);
    elseif(string.sub(args, 1, 4) == "show") then
        GLSavedVars.hiding = nil;
        GLPrint(GETLINK_CONFIG_HIDE_NO);
    elseif(string.sub(args, 1, 4) == "mode") then
        local n = string.sub(args, 6, -1);
        if((n == "w") or (n == "whisper") or (n == "t") or (n == "tell") or (n == "pm")) then
            GLSavedVars.w = true;
            GLPrint(GETLINK_CONFIG_WHISPER);
        else
            GLSavedVars.w = nil;
            GLPrint(GETLINK_CONFIG_SAME);
        end
    elseif(string.sub(args, 1, 7) == "trigger") then
        local n = string.sub(args, 9, -1);
        GLSavedVars.trigger = n;
        GLPrint(string.format(GETLINK_CONFIG_TRIGGER, n));
    else
        GLPrint("GetLink Bot Help :");
        GLPrint(" Options: on, off, ignore, max, hide, show, help");
        GLPrint(" on/off: no other options. Turns the bot on or off.");
        GLPrint(" ignore: option is name (e.g. /glbot ignore guy), toggles ignoring of name on or off.");
        GLPrint(" max: option is new max (e.g. /glbot max 20), changes the max amount of links returned.");
        GLPrint(" hide/show: no other options. Toggles hiding what you link to others on or off.");
        GLPrint(" mode: option is w/whisper. Toggles sending links in whisper or same chat mode.");
        GLPrint(" trigger: option is whatever trigger you want to use. Default is !gl");
    end
end
SLASH_GLBot1 = "/glbot";

GLChatHandler = function()
    if(not GLSavedVars.enabled) then return; end
	local _,_,search = string.find(string.lower(arg1), "^" .. GLSavedVars.trigger .. "%s(.+)");
	if((not search) or (GLSavedVars.ignored[arg2])) then return; end
    
    local chatmode = "WHISPER";
    if(not GLSavedVars.w) then _,_,chatmode = string.find(event, "CHAT_MSG_(.+)"); end
    
    local results, num = GetLink(search);
	if(results) then
		local currentReply = "\20";
		if(num > GLSavedVars.max) then
			SendChatMessage(string.format("\20" .. GETLINK_LIMIT_RESULTS, GLSavedVars.max, num), chatmode, DEFAULT_CHAT_FRAME.defaultLanguage, arg2)
		end
		for i = 1, GLSavedVars.max do
			if(results[i]) then
				local curRepTmp = currentReply .. results[i]
				if(string.len(curRepTmp) < 255) then
					currentReply = "\20" .. curRepTmp;
				else
					SendChatMessage(currentReply, chatmode, DEFAULT_CHAT_FRAME.defaultLanguage, arg2)
					currentReply = "\20" .. results[i];
				end
			end
		end
		if(num == 0) then
			SendChatMessage(string.format("\20" .. GETLINK_NO_RESULTS, search), chatmode, DEFAULT_CHAT_FRAME.defaultLanguage, arg2)
		else
			if(string.len(currentReply) > 1) then
				SendChatMessage(currentReply, chatmode, DEFAULT_CHAT_FRAME.defaultLanguage, arg2)
			end
		end
        local sent = GLSavedVars.max;
        if(sent > num) then sent = num; end
        if(GLSavedVars.hiding) then GLPrint(string.format(GETLINK_HIDDEN_QUERY, arg2, search, sent, num)); end
	end
end

-- event handling --
GLOnLoad = function()
    if(GLSavedVars.enabled) then
		GLFrame:RegisterEvent("CHAT_MSG_GUILD");
		GLFrame:RegisterEvent("CHAT_MSG_WHISPER");
		GLFrame:RegisterEvent("CHAT_MSG_SAY");
		GLFrame:RegisterEvent("CHAT_MSG_RAID");
		GLFrame:RegisterEvent("CHAT_MSG_PARTY");
		GLFrame:RegisterEvent("CHAT_MSG_CHANNEL");
		GLFrame:RegisterEvent("CHAT_MSG_YELL");
    end
    DEFAULT_CHAT_FRAME:AddMessage("GetLink v" .. GLVersion .. " loaded!");
end
GLFrame = CreateFrame("Frame");
GLFrame:SetScript("OnEvent", GLChatHandler);
GLOnLoad();

-- special hook! --
local GLChatFrame_OnEvent = ChatFrame_OnEvent;
ChatFrame_OnEvent = function()
    if(event == "CHAT_MSG_WHISPER_INFORM") then
        if((string.find(arg1, "^\20")) and (GLSavedVars.hiding)) then return; end
    end
    GLChatFrame_OnEvent(event);
end