
InFlight = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceHook-2.0", "AceDB-2.0", "Metrognome-2.0");

function InFlight:OnInitialize()
	self:RegisterEvent("TAXIMAP_OPENED");
	self:RegisterDB("InFlightDB");
	self:RegisterDefaults("profile", {
		confirmflight = false,
		dismount = true, 
		ocb = true,
		fill = "Fill up",
		color = { r = 0.2, g = 0.2, b = 0.2, },
	} );
end

-- setup only when the taxi map is opened the first time
function InFlight:TAXIMAP_OPENED()
	if not self.initbulk then
		self:Hook("TakeTaxiNode");
		self:Hook("TaxiNodeOnButtonEnter");
		self.faction = UnitFactionGroup("player");
		self:RegisterMetro(self.name, self.FlightTime, 0.25, self);
		if not InFlightVars then
			InFlightVars = { };
			self:LoadDefaults();
		end
		self.initbulk = 1;
	end
	if self.db.profile.dismount then
		self:Dismount()
	end
end

-- add time info on the taxi map tooltips
function InFlight:TaxiNodeOnButtonEnter(button)
	self.hooks.TaxiNodeOnButtonEnter.orig(button)
	
	local index = button:GetID()
	if TaxiNodeGetType(index) == "REACHABLE" then
		local source;
		for i = 1, NumTaxiNodes(), 1 do
			if (TaxiNodeGetType(i) == "CURRENT") then
				source = self:ShortenName(TaxiNodeName(i));
				break;
			end
		end
		local destination = self:ShortenName(TaxiNodeName(index));

		if InFlightVars[self.faction][source] and InFlightVars[self.faction][source][destination] then
			local t = InFlightVars[self.faction][source][destination];
			if t == 0 then
				GameTooltip:AddLine("Time: -:--", 0.8, 0.8, 0.8);
			else
				GameTooltip:AddLine(string.format("Time: %d:%02d", t / 60, math.mod(t, 60)), 1, 1, 1)
			end
		else
			GameTooltip:AddLine("Time: -:--", 0.8, 0.8, 0.8);
		end
		GameTooltip:Show();
	end
end

-- main hook to determine when a flight is taken
function InFlight:TakeTaxiNode(slot)
	if TaxiNodeGetType(slot) == "REACHABLE" then
		-- get current location
		for i = 1, NumTaxiNodes(), 1 do
			if (TaxiNodeGetType(i) == "CURRENT") then
				self.source = self:ShortenName(TaxiNodeName(i));
				break;
			end
		end
		self.destination = self:ShortenName(TaxiNodeName(slot));
		if InFlightVars[self.faction][self.source] and 
		   InFlightVars[self.faction][self.source][self.destination] and 
		   InFlightVars[self.faction][self.source][self.destination] ~= 0 then
			self.endTime = InFlightVars[self.faction][self.source][self.destination];
		else
			self.endTime = nil;
		end
		-- create the flight bar if needed
		if self.db.profile.confirmflight then
			self:ConfirmFlight(slot, self.destination, self.endTime);
		else
			self:StartTimer(slot)
		end
	end
end

function InFlight:StartTimer(slot)
	self.totalTime = 0;
	
	if not InFlightBar then
		self:CreateBar();
	end
	InFlightBar.locText:SetText(self.source .. " --> " .. self.destination);
	
	-- start the timers and setup statusbar
	if self.endTime then
		InFlightBar.bar:SetMinMaxValues(0, self.endTime);
		if self.db.profile.fill == "Deplete" then
			InFlightBar.bar:SetValue(self.endTime);
		else
			InFlightBar.bar:SetValue(0);
		end
		if oCB and self.db.profile.ocb then
			local c
			if self.db.profile.fill == "Deplete" then
				c = oCB.db.profile.Mirror.BREATH;
			else
				c = oCB.db.profile.Colors.Casting;
			end
			InFlightBar.bar:SetStatusBarColor(c.r, c.g, c.b, 0.7);
		else
			InFlightBar.bar:SetStatusBarColor(self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b, 0.7);
		end
		InFlightBar.spark:Show();
	else
		InFlightBar.bar:SetMinMaxValues(0,1);
		InFlightBar.bar:SetValue(1);
		if oCB and self.db.profile.ocb then
			if self.db.profile.fill == "Deplete" then
				c = oCB.db.profile.Mirror.BREATH;
			else
				c = oCB.db.profile.Colors.Channel;
			end
			InFlightBar.bar:SetStatusBarColor(c.r, c.g, c.b, 0.7);
		else
			InFlightBar.bar:SetStatusBarColor(0.1, 0.1, 1.0, 0.7);
		end
		InFlightBar.spark:Hide();
	end
	self:StartMetro(self.name);
	self.hooks.TakeTaxiNode.orig(slot)
end

-- in flight onupdate function
function InFlight:FlightTime(difference)
	if not self.active and UnitOnTaxi("player") then
		self.active = 1;
	end
	if not self.active then
		return;
	end
	self.totalTime = self.totalTime + difference;
	InFlightBar:Show();
	InFlightBar.bar:Show();
	if self.active and not UnitOnTaxi("player") then
		if self.endTime and self.totalTime + 15 < self.endTime then
			self:ConfirmSave();
		else
			self:SaveFlightTime();
		end
		InFlightBar:Hide()
		self.endTime = nil;
		self.active = nil;
		self:StopMetro(self.name);
		return;
	end
	if self.endTime then
		if self.totalTime - 2 > self.endTime  then
			InFlightBar.bar:SetMinMaxValues(0,1);
			InFlightBar.bar:SetValue(1);
			if oCB then
				local c = oCB.db.profile.Colors.Channel;
				InFlightBar.bar:SetStatusBarColor(c.r, c.g, c.b, 0.7);
			else
				InFlightBar.bar:SetStatusBarColor(0.1, 0.1, 1.0, 0.7);
			end
			InFlightBar.spark:Hide();
			self.endTime = nil;
		else
			if self.db.profile.fill == "Deplete" then
				InFlightBar.bar:SetValue(self.endTime - self.totalTime);
				InFlightBar.spark:SetPoint("CENTER", InFlightBar.bar, "LEFT", ((self.endTime - self.totalTime) / self.endTime) * InFlightBar.bar:GetWidth(), 0);
				InFlightBar.timeText:SetText(string.format("%s / %s", self:FormatTime(self.endTime - self.totalTime), self:FormatTime(self.endTime)));
			else
				InFlightBar.bar:SetValue(self.totalTime);
				InFlightBar.spark:SetPoint("CENTER", InFlightBar.bar, "LEFT", (self.totalTime / self.endTime) * InFlightBar.bar:GetWidth(), 0);
				InFlightBar.timeText:SetText(string.format("%s / %s", self:FormatTime(self.totalTime), self:FormatTime(self.endTime)));
			end
		end
	else
		InFlightBar.timeText:SetText("Timing - " .. self:FormatTime(self.totalTime));
	end
end

-- confirm save data if the flight ended unexpectingly, workaround for when
--   someone accepts a summon or such
function InFlight:ConfirmSave()
	StaticPopupDialogs["INFLIGHT_CONFIRMSAVE"] = {
		text = TEXT("Your flight has ended earlier than expected.  Save the flight data?"),
		button1 = TEXT(OKAY),
		button2 = TEXT(CANCEL),
		OnAccept = function()
			InFlight:SaveFlightTime();
		end,
		timeout = 20,
		exclusive = 1,
		hideOnEscape = 1,
	};
	StaticPopup_Show("INFLIGHT_CONFIRMSAVE");
end

function InFlight:ConfirmFlight(slot, location, flighttime)
	local texttime = "";
	if flighttime then
		texttime = "This flight will take "..self:FormatTime(flighttime)..".";
	end
	StaticPopupDialogs["INFLIGHT_CONFIRMTAKEOFF"] = {
		text = TEXT(string.format("Are you sure you want to fly to %s? %s", location, texttime)),
		button1 = TEXT(OKAY),
		button2 = TEXT(CANCEL),
		OnAccept = function(data)
			InFlight:StartTimer(data);
		end,
		timeout = 0,
		exclusive = 1,
		hideOnEscape = 1,
	};
	local dialog = StaticPopup_Show("INFLIGHT_CONFIRMTAKEOFF");
	if dialog then
		dialog.data = slot;
	end
end

-- save data
function InFlight:SaveFlightTime()
	if not InFlightVars[self.faction][self.source] then
		InFlightVars[self.faction][self.source] = { };
	end

	InFlightVars[self.faction][self.source][self.destination] = floor(self.totalTime + 0.5);
	self.totalTime = 0;
end

-- create the statusbar for the inflight timer
-- basically a copy and paste of oCB's framework, and it will use oCB's settings if available
function InFlight:CreateBar()
	local frame = CreateFrame("Frame", "InFlightBar", UIParent)
	if not InFlightVars.x then
		frame:SetPoint("CENTER", UIParent, "CENTER", 0, 200);
	else
		frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", InFlightVars.x, InFlightVars.y);
	end
	frame:SetMovable(true)
	frame:EnableMouse(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnDragStart", function() if IsShiftKeyDown() then this:StartMoving() end end)
	frame:SetScript("OnDragStop", function() 
		this:StopMovingOrSizing() 
		local a,b,c,d,e = this:GetPoint();
		InFlightVars.x = floor(d + 0.5);
		InFlightVars.y = floor(e + 0.5);
	end)
	
	frame.bar = CreateFrame("StatusBar", nil, frame)
	frame.spark = frame.bar:CreateTexture(nil, "OVERLAY")
	frame.locText = frame.bar:CreateFontString(nil, "OVERLAY")
	frame.timeText = frame.bar:CreateFontString(nil, "OVERLAY")
	self:LoadOptions()
	self:UpdateLook()
end

function InFlight:UpdateLook()
	local db;
	local f, _ = GameFontHighlightSmall:GetFont();
	local frame = InFlightBar
	
	-- if available, use your oCB settings to customize the flight bar
	if oCB and self.db.profile.ocb then
		if self.db.profile.fill == "Deplete" then
			db = oCB.db.profile["MirrorBar"];
		else
			db = oCB.db.profile["CastingBar"];
		end
	end
	if db then
		frame:SetWidth(db.width+9);
		frame:SetHeight(db.height+10);
		frame:SetBackdrop({
			bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
			edgeFile = oCB.Borders[db.edgeFile], edgeSize = 16,
			insets = {left = 5, right = 5, top = 5, bottom = 5},
		});
		if db.BorderColor then
			frame:SetBackdropBorderColor(db.BorderColor.r, db.BorderColor.g, db.BorderColor.b, 0.8);
		else
			frame:SetBackdropBorderColor(0.8, 0.8, 0.8, 0.8);
		end
		if db.BackColor then
			frame:SetBackdropColor(db.BackColor.r, db.BackColor.g, db.BackColor.b, 0.4);
		else
			frame:SetBackdropColor(0.3, 0.3, 0.3, 0.4);
		end
		
		frame.bar:ClearAllPoints();
		frame.bar:SetPoint("CENTER", frame, "CENTER", 0, 0);
		frame.bar:SetWidth(db.width);
		frame.bar:SetHeight(db.height);
		frame.bar:SetStatusBarTexture(oCB.Textures[db.texture]);
		
		frame.spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark");
		frame.spark:SetWidth(16);
		frame.spark:SetHeight(db.height*2.44);
		frame.spark:SetBlendMode("ADD");
		
	else
		frame:SetWidth(239);
		frame:SetHeight(22);
		frame:SetBackdrop({
			bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
			insets = {left = 5, right = 5, top = 5, bottom = 5},
		});
		frame:SetBackdropBorderColor(0.8, 0.8, 0.8, 0.8);
		frame:SetBackdropColor(0.3, 0.3, 0.3, 0.4);
		
		frame.bar:ClearAllPoints();
		frame.bar:SetPoint("CENTER", frame, "CENTER", 0, 0);
		frame.bar:SetWidth(230);
		frame.bar:SetHeight(12);
		frame.bar:SetStatusBarTexture("Interface\\AddOns\\InFlight\\smooth")
		
		frame.spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
		frame.spark:SetWidth(14)
		frame.spark:SetHeight(12*2.44)
		frame.spark:SetBlendMode("ADD")
	end
	
	frame.timeText:SetJustifyH("CENTER");
	frame.timeText:SetWidth(frame:GetWidth());
	frame.timeText:SetFont(f, 12);
	frame.timeText:ClearAllPoints();
	frame.timeText:SetPoint("CENTER", frame, "CENTER", 0, 0);
	
	frame.locText:SetJustifyH("CENTER");
	frame.locText:SetWidth(frame:GetWidth());
	frame.locText:SetFont(f, 12);
	frame.locText:ClearAllPoints();
	frame.locText:SetPoint("BOTTOM", frame, "TOP", 0, 0);		
end


function InFlight:LoadOptions()
	local db = self.db.profile
	self.opts = {
		type = "group",
		args = {
			header = {
				type = "header",
				name = "InFlight",
				order = 1,
			},
			frame = {
				type = "group",
				name = "Appearance",
				desc = "Change the way the statusbar look and behave.",
				order = 2,
				args = {
					oCB = {
						type = "toggle",
						order = 1,
						name = "Use oCB settings",
						desc = "Use the settings of oCB if available (will override local settings).",
						set = function(v) db.ocb = v; self:UpdateLook(); end,
						get = function() return db.ocb end,
					},
					barfill = {
						type = "text",
						order = 2,
						name = "Bar fill type",
						desc = "Set the fill type of the statusbar.",
						set = function(v) db.fill = v; self:UpdateLook(); end,
						get = function() return db.fill end,
						validate = { "Fill up", "Deplete", },
					},
					barcolor = {
						type = "color",
						order = 3,
						name = "Bar color",
						desc = "Set the color of the fill bar.",
						get = function()
							local v = self.db.profile.color
							return v.r,v.g,v.b
						end,
						set = function(r,g,b) self.db.profile.color = {r=r, g=g, b=b}; InFlightBar.bar:SetStatusBarColor(r, g, b); end,
					}
				},
			},
			dismount = {
				type = "toggle",
				order = 3,
				name = "Auto-dismount",
				desc = "Auto-dismount when talking to a flightmaster.",
				set = function(v) db.dismount = v end,
				get = function() return db.dismount end,
			},
			confirm = {
				type = "toggle",
				order = 4,
				name = "Confirm flight",
				desc = "Show a confirmation window to check if you really want to take this flight.",
				set = function(v) db.confirmflight = v end,
				get = function() return db.confirmflight end,
			},
		},
	};
	InFlight.dewdrop = AceLibrary("Dewdrop-2.0");
	self.dewdrop:Register(InFlightBar,
		'children', function()
				self.dewdrop:FeedAceOptionsTable(self.opts)
			end,
		'cursorX', true,
		'cursorY', true
	);
end

-- shorten name to lighten saved vars
function InFlight:ShortenName(name)
	local comma = ", ";
	local found = string.find(name, comma);
	if found then
		name = string.sub(name, 1, found - 1);
	end
	return name;
end

-- simple time format
function InFlight:FormatTime(secs)
	return string.format("%d:%02d", secs / 60, math.mod(secs, 60));
end

-- dismount function, from FlightMap
function InFlight:Dismount()
	local mountTextures = {
		["Interface\\Icons\\Ability_Mount_BlackDireWolf"] = 1,
		["Interface\\Icons\\Ability_Mount_BlackPanther"] = 1,
		["Interface\\Icons\\Ability_Mount_Charger"] = 1,
		["Interface\\Icons\\Ability_Mount_Dreadsteed"] = 1,
		["Interface\\Icons\\Ability_Mount_JungleTiger"] = 1,
		["Interface\\Icons\\Ability_Mount_Kodo_01"] = 1,
		["Interface\\Icons\\Ability_Mount_Kodo_02"] = 1,
		["Interface\\Icons\\Ability_Mount_Kodo_03"] = 1,
		["Interface\\Icons\\Ability_Mount_MechaStrider"] = 1,
		["Interface\\Icons\\INV_Misc_Horn_01"] = 1,
		["Interface\\Icons\\Ability_Mount_MountainRam"] = 1,
		["Interface\\Icons\\Spell_Nature_Swiftness"] = 1,
		["Interface\\Icons\\Ability_Mount_NightmareHorse"] = 1,
		["Interface\\Icons\\Ability_Mount_PinkTiger"] = 1,
		["Interface\\Icons\\Ability_Mount_Raptor"] = 1,
		["Interface\\Icons\\Ability_Mount_RidingHorse"] = 1,
		["Interface\\Icons\\Ability_Mount_Undeadhorse"] = 1,
		["Interface\\Icons\\Ability_Mount_WhiteDireWolf"] = 1,
		["Interface\\Icons\\Ability_Mount_WhiteTiger"] = 1,
	};
	for i = 0, 15, 1 do
		local id, isAura = GetPlayerBuff(i, "HELPFUL");
		local texture = GetPlayerBuffTexture(id);
		if isAura and mountTextures[texture] then
			CancelPlayerBuff(id);
			return;
		end
	end
end